@@ -1,15 +1,4 @@
 
-# setequal function for Nodes
-nodesetequal <- function(A,B) {
-  AinB <- all(sapply(A, function(a) {
-    return(any(sapply(B, function(b){identical(a,b)})))
-  }))  
-  BinA <- all(sapply(B, function(b) {
-    return(any(sapply(A, function(a){identical(a,b)})))
-  }))
-  return(AinB & BinA)
-}
-
 # tests of graph creation
 test_that("incorrect node and edge types are rejected", {
   n1 <- Node$new()
@@ -112,16 +101,16 @@ test_that("adjacency matrix has correct properties", {
   e1 <- Edge$new(n1,n2)
   G <- Graph$new(V=list(n1,n2),E=list(e1))
   A <- G$graph_adjacency_matrix()
-  expect_true(is.null(dimnames(A))) 
+  expect_null(dimnames(A)) 
   n1 <- Node$new("n1")
   n2 <- Node$new("n2")
   e1 <- Edge$new(n1,n2)
   G <- Graph$new(V=list(n1,n2),E=list(e1))
   A <- G$graph_adjacency_matrix()
   dn <- dimnames(A)
-  expect_equal(names(dn), c("out.node", "in.node"))  
-  expect_equal(dn$out.node, c("n1", "n2"))
-  expect_equal(dn$in.node, c("n1", "n2"))
+  expect_setequal(names(dn), c("out.node", "in.node"))  
+  expect_setequal(dn$out.node, c("n1", "n2"))
+  expect_setequal(dn$in.node, c("n1", "n2"))
   expect_equal(sum(A-matrix(c(0,1,0,1),nrow=2)),0)
   # binary
   n1 <- Node$new("n1")
@@ -243,10 +232,10 @@ test_that("Fig 1.1.1 from Gross & Yellen (2013)", {
   # neighbours
   XX <- Node$new("XX")
   expect_error(G$neighbours(XX), class="not_in_graph")
-  expect_true(nodesetequal(G$neighbours(u),list(v)))
-  expect_true(nodesetequal(G$neighbours(v),list(u,w,x)))
-  expect_true(nodesetequal(G$neighbours(w),list(v,x)))
-  expect_true(nodesetequal(G$neighbours(x),list(v,w)))
+  expect_R6setequal(G$neighbours(u),list(v))
+  expect_R6setequal(G$neighbours(v),list(u,w,x))
+  expect_R6setequal(G$neighbours(w),list(v,x))
+  expect_R6setequal(G$neighbours(x),list(v,w))
   # connected
   expect_true(G$is_connected())
   # cycle
@@ -255,3 +244,19 @@ test_that("Fig 1.1.1 from Gross & Yellen (2013)", {
   expect_false(G$is_tree())
 })
 
+test_that("Fig 1.1.1 from Gross & Yellen (2013) is drawn correctly", {
+  # the graph
+  u <- Node$new("u")
+  v <- Node$new("v")
+  w <- Node$new("w")
+  x <- Node$new("x")
+  a <- Edge$new(u,v,"a")
+  b <- Edge$new(v,u,"b")
+  c <- Edge$new(x,x,"c")
+  d <- Edge$new(x,w,"d")
+  e <- Edge$new(x,v,"e")
+  f <- Edge$new(w,v,"f")
+  G <- Graph$new(V=list(u,v,w,x), E=list(a,b,c,d,e,f))
+  # draw it
+  expect_silent(DOT <- G$as_DOT())
+})
